@@ -238,8 +238,13 @@ struct col_info
 	pgNAME		schema_name;
 	pgNAME		table_name;
 	OID		table_oid;
+	int		table_info;
 	time_t		acc_time;
 };
+enum {
+	TBINFO_HASOIDS	 = 1L
+	,TBINFO_HASSUBCLASS = (1L << 1)
+};
 #define free_col_info_contents(coli) \
 { \
 	if (NULL != coli->result) \
@@ -315,7 +320,6 @@ struct ConnectionClass_
 	/* for per statement rollback */
 	char		internal_svp;		/* is set? */
 	char		internal_op;		/* operation being executed as to internal savepoint */
-	unsigned char	lock_CC_for_rb;
 	unsigned char	rbonerr;
 	unsigned char	opt_in_progress;
 	unsigned char	opt_previous;
@@ -479,6 +483,20 @@ enum {
 #define	NO_TRANS		1L
 #define	CONN_DEAD		(1L << 1) /* connection is no longer valid */
 
+/*
+ * This is a libpq notice receiver callback, for handling incoming NOTICE
+ * messages while processing a query.
+ */
+typedef struct
+{
+	ConnectionClass *conn;
+	const char *comment;
+	QResultClass *res;
+	StatementClass *stmt;
+} notice_receiver_arg;
+
+void receive_libpq_notice(void *arg, const PGresult *pgres);
+
 /*
  *	internal savepoint related
  */
