@@ -354,7 +354,8 @@ MYLOG(DETAIL_LOG_LEVEL, "hlen=" FORMAT_SSIZE_T "\n", hlen);
 			INI_LOWERCASEIDENTIFIER "=%d;"
 			"%s"		/* INI_PQOPT */
 			"%s"		/* INIKEEPALIVE TIME/INTERVAL */
-			INI_NUMERIC_AS "=%d;"
+			ABBR_NUMERIC_AS "=%d;"
+			INI_OPTIONAL_ERRORS "=%d;"
 #ifdef	_HANDLE_ENLIST_IN_DTC_
 			INI_XAOPT "=%d"	/* XAOPT */
 #endif /* _HANDLE_ENLIST_IN_DTC_ */
@@ -388,6 +389,7 @@ MYLOG(DETAIL_LOG_LEVEL, "hlen=" FORMAT_SSIZE_T "\n", hlen);
 			,makeBracketConnectString(ci->pqopt_in_str, &pqoptStr, ci->pqopt, INI_PQOPT)
 			,makeKeepaliveConnectString(keepaliveStr, sizeof(keepaliveStr), ci, FALSE)
 			,ci->numeric_as
+			,ci->optional_errors
 #ifdef	_HANDLE_ENLIST_IN_DTC_
 			,ci->xa_opt
 #endif /* _HANDLE_ENLIST_IN_DTC_ */
@@ -444,6 +446,8 @@ MYLOG(DETAIL_LOG_LEVEL, "hlen=" FORMAT_SSIZE_T "\n", hlen);
 			flag |= BIT_USESERVERSIDEPREPARE;
 		if (ci->lower_case_identifier)
 			flag |= BIT_LOWERCASEIDENTIFIER;
+		if (ci->optional_errors)
+			flag |= BIT_OPTIONALERRORS;
 
 		if (ci->sslmode[0])
 		{
@@ -463,7 +467,7 @@ MYLOG(DETAIL_LOG_LEVEL, "hlen=" FORMAT_SSIZE_T "\n", hlen);
 				ABBR_EXTRASYSTABLEPREFIXES "=%s;"
 				"%s"		/* ABBR_PQOPT */
 				"%s"		/* ABBRKEEPALIVE TIME/INTERVAL */
-				INI_NUMERIC_AS "=%d;"
+				ABBR_NUMERIC_AS "=%d;"
 #ifdef	_HANDLE_ENLIST_IN_DTC_
 				"%s"
 #endif /* _HANDLE_ENLIST_IN_DTC_ */
@@ -565,6 +569,7 @@ unfoldCXAttribute(ConnInfo *ci, const char *value)
 	ci->bytea_as_longvarbinary = (char)((flag & BIT_BYTEAASLONGVARBINARY) != 0);
 	ci->use_server_side_prepare = (char)((flag & BIT_USESERVERSIDEPREPARE) != 0);
 	ci->lower_case_identifier = (char)((flag & BIT_LOWERCASEIDENTIFIER) != 0);
+	ci->optional_errors = (char)((flag & BIT_OPTIONALERRORS) != 0);
 }
 
 BOOL
@@ -662,7 +667,7 @@ copyConnAttributes(ConnInfo *ci, const char *attribute, const char *value)
 		ci->true_is_minus1 = atoi(value);
 	else if (stricmp(attribute, INI_INT8AS) == 0)
 		ci->int8_as = atoi(value);
-	else if (stricmp(attribute, INI_NUMERIC_AS) == 0)
+	else if (stricmp(attribute, INI_NUMERIC_AS) == 0 || stricmp(attribute, ABBR_NUMERIC_AS) == 0)
 		ci->numeric_as = atoi(value);
 	else if (stricmp(attribute, INI_BYTEAASLONGVARBINARY) == 0 || stricmp(attribute, ABBR_BYTEAASLONGVARBINARY) == 0)
 		ci->bytea_as_longvarbinary = atoi(value);
@@ -674,6 +679,12 @@ copyConnAttributes(ConnInfo *ci, const char *attribute, const char *value)
 		ci->keepalive_idle = atoi(value);
 	else if (stricmp(attribute, INI_KEEPALIVEINTERVAL) == 0 || stricmp(attribute, ABBR_KEEPALIVEINTERVAL) == 0)
 		ci->keepalive_interval = atoi(value);
+	else if (stricmp(attribute, INI_BATCHSIZE) == 0 || stricmp(attribute, ABBR_BATCHSIZE) == 0)
+		ci->batch_size = atoi(value);
+	else if (stricmp(attribute, INI_OPTIONAL_ERRORS) == 0 || stricmp(attribute, ABBR_OPTIONAL_ERRORS) == 0)
+		ci->optional_errors = atoi(value);
+	else if (stricmp(attribute, INI_IGNORETIMEOUT) == 0 || stricmp(attribute, ABBR_IGNORETIMEOUT) == 0)
+		ci->ignore_timeout = atoi(value);
 	else if (stricmp(attribute, INI_SSLMODE) == 0 || stricmp(attribute, ABBR_SSLMODE) == 0)
 	{
 		switch (value[0])
@@ -797,6 +808,7 @@ getCiDefaults(ConnInfo *ci)
 	ci->true_is_minus1 = DEFAULT_TRUEISMINUS1;
 	ci->int8_as = DEFAULT_INT8AS;
 	ci->numeric_as = DEFAULT_NUMERIC_AS;
+	ci->optional_errors = DEFAULT_OPTIONAL_ERRORS;
 	ci->bytea_as_longvarbinary = DEFAULT_BYTEAASLONGVARBINARY;
 	ci->use_server_side_prepare = DEFAULT_USESERVERSIDEPREPARE;
 	ci->lower_case_identifier = DEFAULT_LOWERCASEIDENTIFIER;
@@ -1026,9 +1038,12 @@ MYLOG(0, "drivername=%s\n", drivername);
 	if (SQLGetPrivateProfileString(DSN, INI_INT8AS, NULL_STRING, temp, sizeof(temp), ODBC_INI) > 0)
 		ci->int8_as = atoi(temp);
 
-	if (SQLGetPrivateProfileString(DSN, INI_NUMERIC_AS, NULL_STRING, temp, sizeof(temp), ODBC_INI) > 0)
+	if (SQLGetPrivateProfileString(DSN, ABBR_NUMERIC_AS, NULL_STRING, temp, sizeof(temp), ODBC_INI) > 0)
 		ci->numeric_as = atoi(temp);
 
+	if (SQLGetPrivateProfileString(DSN, INI_OPTIONAL_ERRORS, NULL_STRING, temp, sizeof(temp), ODBC_INI) > 0)
+		ci->optional_errors = atoi(temp);
+
 	if (SQLGetPrivateProfileString(DSN, INI_BYTEAASLONGVARBINARY, NULL_STRING, temp, sizeof(temp), ODBC_INI) > 0)
 		ci->bytea_as_longvarbinary = atoi(temp);
 
@@ -1044,6 +1059,11 @@ MYLOG(0, "drivername=%s\n", drivername);
 	if (SQLGetPrivateProfileString(DSN, INI_KEEPALIVEINTERVAL, NULL_STRING, temp, sizeof(temp), ODBC_INI) > 0)
 		if (0 == (ci->keepalive_interval = atoi(temp)))
 			ci->keepalive_interval = -1;
+	if (SQLGetPrivateProfileString(DSN, INI_BATCHSIZE, NULL_STRING, temp, sizeof(temp), ODBC_INI) > 0)
+		if (0 == (ci->batch_size = atoi(temp)))
+			ci->batch_size = DEFAULT_BATCH_SIZE;
+	if (SQLGetPrivateProfileString(DSN, INI_IGNORETIMEOUT, NULL_STRING, temp, sizeof(temp), ODBC_INI) > 0)
+		ci->ignore_timeout = atoi(temp);
 
 	if (SQLGetPrivateProfileString(DSN, INI_SSLMODE, NULL_STRING, temp, sizeof(temp), ODBC_INI) > 0)
 		STRCPY_FIXED(ci->sslmode, temp);
@@ -1288,7 +1308,12 @@ writeDSNinfo(const ConnInfo *ci)
 								 ODBC_INI);
 	ITOA_FIXED(temp, ci->numeric_as);
 	SQLWritePrivateProfileString(DSN,
-								 INI_NUMERIC_AS,
+								 ABBR_NUMERIC_AS,
+								 temp,
+								 ODBC_INI);
+	ITOA_FIXED(temp, ci->optional_errors);
+	SQLWritePrivateProfileString(DSN,
+								 INI_OPTIONAL_ERRORS,
 								 temp,
 								 ODBC_INI);
 	SPRINTF_FIXED(temp, "%x", getExtraOptions(ci));
@@ -1325,6 +1350,16 @@ writeDSNinfo(const ConnInfo *ci)
 								 INI_KEEPALIVEINTERVAL,
 								 temp,
 								 ODBC_INI);
+	ITOA_FIXED(temp, ci->batch_size);
+	SQLWritePrivateProfileString(DSN,
+								 INI_BATCHSIZE,
+								 temp,
+								 ODBC_INI);
+	ITOA_FIXED(temp, ci->ignore_timeout);
+	SQLWritePrivateProfileString(DSN,
+								 INI_IGNORETIMEOUT,
+								 temp,
+								 ODBC_INI);
 #ifdef	_HANDLE_ENLIST_IN_DTC_
 	ITOA_FIXED(temp, ci->xa_opt);
 	SQLWritePrivateProfileString(DSN, INI_XAOPT, temp, ODBC_INI);
@@ -1722,6 +1757,7 @@ CC_conninfo_init(ConnInfo *conninfo, UInt4 option)
 	conninfo->true_is_minus1 = -1;
 	conninfo->int8_as = -101;
 	conninfo->numeric_as = DEFAULT_NUMERIC_AS;
+	conninfo->optional_errors = -1;
 	conninfo->bytea_as_longvarbinary = -1;
 	conninfo->use_server_side_prepare = -1;
 	conninfo->lower_case_identifier = -1;
@@ -1735,6 +1771,8 @@ CC_conninfo_init(ConnInfo *conninfo, UInt4 option)
 	conninfo->disable_keepalive = -1;
 	conninfo->keepalive_idle = -1;
 	conninfo->keepalive_interval = -1;
+	conninfo->batch_size = DEFAULT_BATCH_SIZE;
+	conninfo->ignore_timeout = DEFAULT_IGNORETIMEOUT;
 	conninfo->wcs_debug = -1;
 #ifdef	_HANDLE_ENLIST_IN_DTC_
 	conninfo->xa_opt = -1;
@@ -1819,6 +1857,7 @@ CC_copy_conninfo(ConnInfo *ci, const ConnInfo *sci)
 	CORR_VALCPY(true_is_minus1);
 	CORR_VALCPY(int8_as);
 	CORR_VALCPY(numeric_as);
+	CORR_VALCPY(optional_errors);
 	CORR_VALCPY(bytea_as_longvarbinary);
 	CORR_VALCPY(use_server_side_prepare);
 	CORR_VALCPY(lower_case_identifier);
@@ -1833,6 +1872,8 @@ CC_copy_conninfo(ConnInfo *ci, const ConnInfo *sci)
 	CORR_VALCPY(extra_opts);
 	CORR_VALCPY(keepalive_idle);
 	CORR_VALCPY(keepalive_interval);
+	CORR_VALCPY(batch_size);
+	CORR_VALCPY(ignore_timeout);
 #ifdef	_HANDLE_ENLIST_IN_DTC_
 	CORR_VALCPY(xa_opt);
 #endif
