@@ -457,7 +457,15 @@ SC_Constructor(ConnectionClass *conn)
 		rv->callbacks = NULL;
 		GetDataInfoInitialize(SC_get_GDTI(rv));
 		PutDataInfoInitialize(SC_get_PDTI(rv));
+		rv->use_server_side_prepare = conn->connInfo.use_server_side_prepare;
 		rv->lock_CC_for_rb = FALSE;
+		// for batch execution
+		memset(&rv->stmt_deffered, 0, sizeof(rv->stmt_deffered));
+		if ((rv->batch_size = conn->connInfo.batch_size) < 1)
+			rv->batch_size = 1;
+		rv->exec_type = DIRECT_EXEC;
+		rv->count_of_deffered = 0;
+		rv->has_notice = 0;
 		INIT_STMT_CS(rv);
 	}
 	return rv;
@@ -506,6 +514,8 @@ SC_Destructor(StatementClass *self)
 	cancelNeedDataState(self);
 	if (self->callbacks)
 		free(self->callbacks);
+	if (!PQExpBufferDataBroken(self->stmt_deffered))
+		termPQExpBuffer(&self->stmt_deffered);
 
 	DELETE_STMT_CS(self);
 	free(self);
@@ -687,10 +697,12 @@ SC_initialize_stmts(StatementClass *self, BOOL initializeOriginal)
 {
 	ProcessedStmt *pstmt;
 	ProcessedStmt *next_pstmt;
+	ConnectionClass *conn = SC_get_conn(self);
 
 	if (self->lock_CC_for_rb)
 	{
-		LEAVE_CONN_CS(SC_get_conn(self));
+		if (conn)
+			LEAVE_CONN_CS(conn);
 		self->lock_CC_for_rb = FALSE;
 	}
 	if (initializeOriginal)
@@ -721,6 +733,8 @@ SC_initialize_stmts(StatementClass *self, BOOL initializeOriginal)
 		self->join_info = 0;
 		SC_init_parse_method(self);
 		SC_init_discard_output_params(self);
+		if (conn)
+			self->use_server_side_prepare = conn->connInfo.use_server_side_prepare;
 	}
 	if (self->stmt_with_params)
 	{
@@ -732,6 +746,7 @@ SC_initialize_stmts(StatementClass *self, BOOL initializeOriginal)
 		free(self->load_statement);
 		self->load_statement = NULL;
 	}
+	self->has_notice = 0;
 
 	return 0;
 }
@@ -1275,9 +1290,13 @@ SC_create_errorinfo(const StatementClass *self, PG_ErrorInfo *pgerror_fail_safe)
 			if (NULL != sqlstate && strnicmp(res->sqlstate, "00", 2) == 0)
 				continue;
 			sqlstate = res->sqlstate;
+			if (!QR_command_maybe_successful(res))
+				loopend = TRUE;
+			/*
 			if ('0' != sqlstate[0] ||
 			    '1' < sqlstate[1])
 				loopend = TRUE;
+			*/
 		}
 		if (NULL != res->message)
 		{
@@ -1833,10 +1852,19 @@ SC_execute(StatementClass *self)
 	BOOL		use_extended_protocol;
 	int		func_cs_count = 0, i;
 	BOOL		useCursor, isSelectType;
+	int		errnum_sav = STMT_OK, errnum;
+	char		*errmsg_sav = NULL;
+	SQLULEN		stmt_timeout;
 
 	conn = SC_get_conn(self);
 	ci = &(conn->connInfo);
 
+	errnum_sav = SC_get_errornumber(self);
+	errmsg_sav = SC_get_errormsg(self);
+	if (NULL != errmsg_sav)
+		errmsg_sav = strdup(errmsg_sav);
+	SC_set_error(self, STMT_OK, NULL, __FUNCTION__);
+
 	/* Begin a transaction if one is not already in progress */
 
 	/*
@@ -1911,15 +1939,16 @@ SC_execute(StatementClass *self)
 	 * If the session query timeout setting differs from the statement one,
 	 * change it.
 	 */
-	if (conn->stmt_timeout_in_effect != self->options.stmt_timeout)
+	stmt_timeout = conn->connInfo.ignore_timeout ? 0 : self->options.stmt_timeout;
+	if (conn->stmt_timeout_in_effect != stmt_timeout)
 	{
 		char query[64];
 
 		SPRINTF_FIXED(query, "SET statement_timeout = %d",
-				 (int) self->options.stmt_timeout * 1000);
+				 (int) stmt_timeout * 1000);
 		res = CC_send_query(conn, query, NULL, 0, NULL);
 		if (QR_command_maybe_successful(res))
-			conn->stmt_timeout_in_effect = self->options.stmt_timeout;
+			conn->stmt_timeout_in_effect = stmt_timeout;
 		QR_Destructor(res);
 	}
 
@@ -2072,9 +2101,17 @@ SC_execute(StatementClass *self)
 		if (0 < SC_get_errornumber(self))
 			;
 		else if (was_ok)
-			SC_set_errornumber(self, STMT_OK);
+		{
+			if (self->has_notice &&
+			    0 == SC_get_errornumber(self))
+				SC_set_errornumber(self, STMT_INFO_ONLY);
+		}
 		else if (was_nonfatal)
-			SC_set_errornumber(self, STMT_INFO_ONLY);
+		{
+			self->has_notice = 1;
+			if (0 == SC_get_errornumber(self))
+				SC_set_errornumber(self, STMT_INFO_ONLY);
+		}
 		else
 			SC_set_errorinfo(self, res, 0);
 		/* set cursor before the first tuple in the list */
@@ -2259,15 +2296,26 @@ cleanup:
 	if (CONN_DOWN != conn->status)
 		conn->status = oldstatus;
 	/* self->status = STMT_FINISHED; */
-	if (SC_get_errornumber(self) == STMT_OK)
+	errnum = SC_get_errornumber(self);
+	if (errnum_sav > STMT_OK)
+		SC_set_error(self, errnum_sav, errmsg_sav, NULL);
+	else if (errnum > STMT_OK)
+		;
+	else if (errnum_sav < STMT_OK)
+		SC_set_error(self, errnum_sav, errmsg_sav, NULL);
+	if (NULL != errmsg_sav)
+		free(errmsg_sav);
+
+	if (errnum == STMT_OK)
 		return SQL_SUCCESS;
-	else if (SC_get_errornumber(self) < STMT_OK)
+	else if (errnum < STMT_OK)
 		return SQL_SUCCESS_WITH_INFO;
 	else
 	{
-		if (!SC_get_errormsg(self) || !SC_get_errormsg(self)[0])
+		char *errmsg = SC_get_errormsg(self);
+		if (!errmsg || !errmsg[0])
 		{
-			if (STMT_NO_MEMORY_ERROR != SC_get_errornumber(self))
+			if (STMT_NO_MEMORY_ERROR != errnum)
 				SC_set_errormsg(self, "Error while executing the query");
 			SC_log_error(func, NULL, self);
 		}
@@ -2453,6 +2501,24 @@ static void log_params(int nParams, const Oid *paramTypes, const UCHAR * const *
 	}
 }
 
+static
+QResultClass *add_libpq_notice_receiver(StatementClass *stmt, notice_receiver_arg *nrarg)
+{
+	QResultClass *res = NULL, *newres = NULL;
+
+	if (stmt->curr_param_result)
+		for (res = SC_get_Result(stmt); NULL != res && NULL != res->next; res = res->next);
+	if (!res)
+		newres = res = QR_Constructor();
+	nrarg->conn = SC_get_conn(stmt);
+	nrarg->comment = __FUNCTION__;
+	nrarg->res = res;
+	nrarg->stmt = stmt;
+	PQsetNoticeReceiver(nrarg->conn->pqconn, receive_libpq_notice, nrarg);
+
+	return newres;
+}
+
 static QResultClass *
 libpq_bind_and_exec(StatementClass *stmt)
 {
@@ -2470,6 +2536,7 @@ libpq_bind_and_exec(StatementClass *stmt)
 	QResultClass *res = NULL;
 	char	   *cmdtag;
 	char	   *rowcount;
+	notice_receiver_arg	nrarg;
 
 	if (!RequestStart(stmt, conn, func))
 		return NULL;
@@ -2526,6 +2593,8 @@ libpq_bind_and_exec(StatementClass *stmt)
 		pstmt = stmt->processed_statements;
 		QLOG(0, "PQexecParams: %p '%s' nParams=%d\n", conn->pqconn, pstmt->query, nParams);
 		log_params(nParams, paramTypes, (const UCHAR * const *) paramValues, paramLengths, paramFormats, resultFormat);
+		/* set notice receiver */
+		newres = add_libpq_notice_receiver(stmt, &nrarg);
 		pgres = PQexecParams(conn->pqconn,
 							 pstmt->query,
 							 nParams,
@@ -2551,27 +2620,20 @@ libpq_bind_and_exec(StatementClass *stmt)
 		/* already prepared */
 		QLOG(0, "PQexecPrepared: %p plan=%s nParams=%d\n", conn->pqconn, plan_name, nParams);
 		log_params(nParams, paramTypes, (const UCHAR * const *) paramValues, paramLengths, paramFormats, resultFormat);
+		/* set notice receiver */
+		newres = add_libpq_notice_receiver(stmt, &nrarg);
 		pgres = PQexecPrepared(conn->pqconn,
 							   plan_name, 	/* portal name == plan name */
 							   nParams,
 							   (const char **) paramValues, paramLengths, paramFormats,
 							   resultFormat);
 	}
-	if (stmt->curr_param_result)
+	/* reset notice receiver */
+	PQsetNoticeReceiver(conn->pqconn, receive_libpq_notice, NULL);
+	if (!(res = nrarg.res))
 	{
-		for (res = SC_get_Result(stmt); NULL != res && NULL != res->next; res = res->next) ;
-	}
-	else
-		res = NULL;
-
-	if (!res)
-	{
-		newres = res = QR_Constructor();
-		if (!res)
-		{
-			SC_set_error(stmt, STMT_NO_MEMORY_ERROR, "Out of memory while allocating result set", func);
-			goto cleanup;
-		}
+		SC_set_error(stmt, STMT_NO_MEMORY_ERROR, "Out of memory while allocating result set", func);
+		goto cleanup;
 	}
 
 	/* 3. Receive results */
