@@ -23,9 +23,6 @@ int main(int argc, char **argv)
 {
 	int rc;
 	HSTMT hstmt = SQL_NULL_HSTMT;
-	char sql[100000];
-	char *sqlend;
-	int i;
 	char *param1;
 	SQLLEN cbParam1;
 	char *param2;
@@ -109,4 +106,6 @@ int main(int argc, char **argv)
 
 	/* Clean up */
 	test_disconnect();
+
+	return 0;
 }
